﻿using System;
using System.Drawing;
using System.Windows.Forms;
using System.Collections.Generic;

// installare il font allegato: matrix code nfi.ttf

namespace MatrixGDI
{
    public partial class Form1 : Form
    {
        public struct Colonna 
        {
            public int sotto; // coordinata
            public int sinistra; // coordinata
            public string text; // stringa
            public int lung; // lunghezza stringa
            public Font font; // tipo font
            public int interlinea; // spaziatura verticale
            public int vel; // velocità
        }
        string mtx = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz1234567890+-.,<|!$%&/()=^*_:;"; // dove pescare i caratteri
        List<Brush> pennelli = new List<Brush>(); // lista dei colori
        Random rnd = new Random();
        static int nColonne = 500; // colonne visualizzate, aumentare o diminuire a piacere
        Colonna[] colonne = new Colonna[nColonne]; // conterrà i dati per ogni colonna
        Font[] font = new Font[4]; // conterrà i 4 font

        public Form1()
        {
            InitializeComponent();
            font[0] = new Font("Matrix Code NFI", 20, FontStyle.Regular);
            font[1] = new Font("Matrix Code NFI", 14, FontStyle.Bold);
            font[2] = new Font("Matrix Code NFI", 10, FontStyle.Bold);
            font[3] = new Font("Matrix Code NFI", 08, FontStyle.Bold);

            pennelli.Add(new SolidBrush(Color.FromArgb(255, 255, 255))); // bianco
            for (int verde = 255; verde > 30; verde -= 20) // se invece di -=20, si inserisce -=10 le colonne si allungano
                pennelli.Add(new SolidBrush(Color.FromArgb(0, verde, 0))); // sfumatura di verdi
        }

        private void Form1_Shown(object sender, EventArgs e)
        {
            for (int c = 0; c < nColonne; c++) // inizializzazione per tutte le colonne
            {
                int nCaratteri = c % pennelli.Count  * 2; // tutte le lunghezze ammesse
                for (int car = 0; car < nCaratteri + 1; car++) // tra 1 e il doppio dei colori disponibili
                    colonne[c].text +=  mtx[rnd.Next(0, mtx.Length - 1)]; // più un carattere tra i disponibili
 
                colonne[c].sotto = rnd.Next(-800, 0); // una posizione tra -800 e 0
                colonne[c].sinistra = rnd.Next(0, this.ClientSize.Width - 18); // una posizione tra 0 e la larghezza dello schermo
                colonne[c].lung = colonne[c].text.Length - 1; // quanto è lunga la stringa
                colonne[c].font = font[rnd.Next(0, 4)]; // tipo di font, cambia solo la dimensione
                colonne[c].interlinea = Convert.ToInt16(colonne[c].font.Size) + 1; // spaziatura tra i caratteri in verticale
                colonne[c].vel = rnd.Next(4, 12); // usato per incrementare la coordinata Y, min 4, max 11
            }
        }

        private void Form1_Paint(object sender, PaintEventArgs e)
        {
            e.Graphics.TextRenderingHint = System.Drawing.Text.TextRenderingHint.AntiAlias;
            
            for (int c = 0; c < nColonne; c++) // scrittura di tutte le colonne
            {
                double colore = 0.5; // viene convertito in int, il bianco verrà usato una volta i verdi 2 volte
                for (int r1 = 0; r1 < colonne[c].lung; r1++)  // scrittura di tutti i caratteri, la stringa con un carattere non viene stampata perché ha lunghezza zero
                {
                   e.Graphics.DrawString(colonne[c].text[r1].ToString(), colonne[c].font, pennelli[Convert.ToInt16(colore)], colonne[c].sinistra, colonne[c].sotto - r1 * colonne[c].interlinea);
                   colore += 0.5;
                   if (colore > pennelli.Count - 1) colore = pennelli.Count - 1;
                }
            }
        }

        private void timer1_Tick(object sender, EventArgs e)
        {
            for (int c = 0; c < nColonne; c++)
            {   // rotazione carattere, aggiungo nuovo e tolgo l'ultimo
                colonne[c].text = mtx[rnd.Next(0, mtx.Length - 1)] + colonne[c].text.Substring(0, colonne[c].lung);
                colonne[c].sotto += colonne[c].vel; // incremento coordinata
                if (colonne[c].sotto > this.ClientSize.Height + 500) // aggiorna i parametri della colonna che è uscita dallo schermo
                {
                    colonne[c].sotto = rnd.Next(-800,0); // riparte da sopra fuori schermo
                    colonne[c].sinistra = rnd.Next(0, this.ClientSize.Width - 18); // una nuova posizione X a caso
                    colonne[c].font = font[rnd.Next(0, 4)]; // un nuovo font a caso (solo dimensione)
                    colonne[c].interlinea = Convert.ToInt16(colonne[c].font.Size) + 1; // aggiorna l'interlinea in base alla dimensione del font
                }
            }
            this.Invalidate(); // scatena il paint
        }

        private void Form1_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.F)
            {
                if (this.FormBorderStyle == FormBorderStyle.Sizable)
                {
                    this.FormBorderStyle = FormBorderStyle.None;
                    this.WindowState = FormWindowState.Maximized;
                }
                else
                    this.FormBorderStyle = FormBorderStyle.Sizable;
            }

            if (e.KeyCode == Keys.Escape)
                    this.Close();
        }
    }
}
